import processing.core.*; 

import gifAnimation.*; 
import processing.video.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class gif_recorder extends PApplet {

// http://www.extrapixel.ch/processing/gifAnimation/



// Variable for capture device
Capture video;
GifMaker gifExport;
InteractiveGif gifPlayer;

Boolean bRecording = false;
Boolean bLiveView = true;
Boolean bDragged = false;
int numFrames = 0;
String instructions = "Click to record";

public void setup() {
  size(320, 240);
  frameRate( 10 );
  textFont(createFont("Arial", 18));
  // Uses the default video input, see the reference if this causes an error
  video = new Capture(this, width, height);
  video.start();
}

public void draw() {
  background(0);
  if(!bLiveView && !bRecording) {
    if(gifPlayer.bLoaded) {
      if(bDragged) {
        gifPlayer.mouseOver( mouseX, mouseY );
      } else {
        gifPlayer.bActive = false; 
      }
      gifPlayer.render();
    }
  }
  
  if (bLiveView && video.available()) {
    video.read();
    video.loadPixels(); 
    image( video, 0, 0 );
    if(bRecording) {
      gifExport.setDelay(1);
      gifExport.addFrame();
      numFrames ++;
      //fill(255, 0, 0);
      //rect(0,0,34,22);
      //fill(255);
      //text(Integer.toString(numFrames), 8, 18 );
      instructions = "Click to finish - "+Integer.toString(numFrames);
    }
  }
  fill(0);
  if(bRecording) fill(255, 0, 0);
  float textw = textWidth(instructions)+8;
  rect( 4, height - 32, textw, 22);
  fill(255);
  text(instructions, 8, height - 14 );
}

public void stop() {
  if(bRecording) {
    gifExport.finish();
  }
  super.stop();
}

public void mouseDragged() {
  println("mouse dragged");
  bDragged = true;
}

public void mouseClicked() {
  println("mouse pressed");
  
  if(!bDragged) {
    if(!bLiveView && !bRecording) {
      bLiveView = true;
      instructions = "Click to record";
      //return;
    } else if(bRecording) {
      bRecording = false;
      bLiveView = false;
      instructions = "Click and drag to scrub / click to record";
      gifExport.finish();
      gifPlayer.load(this, gifPlayer.filename );
      gifPlayer.play();
    } else {
      instructions = "Click to finish";
      createExporter();
      bRecording = true;
    }
  }
  bDragged = false;
}

public void mouseReleased() {
  println("mouse released");
  bDragged = false;
}

public void createExporter() {
  String uniqueID = "gifs/export_"+getUniqueID()+".gif";
  gifExport = new GifMaker(this, uniqueID);
  gifExport.setRepeat(0); // make it an "endless" animation
  gifPlayer = new InteractiveGif();
  gifPlayer.filename = uniqueID;
  
  numFrames = 0;
}

public String getUniqueID() {
  return Integer.toString(month()+day()+year()+hour()+minute()+second());
}

class InteractiveGif {
  PImage[] frames;
  String filename;
  Gif file;
  Boolean bActive;
  float x, y, w, h;
  float mousePct;
  int frameIndex;
  Boolean bLoaded;
  
  InteractiveGif() {
    bActive = false;
    mousePct = 0.0f;
    frameIndex = 0;
    bLoaded = true;
  }
  public void load(PApplet parent, String location ) {
    filename = location;
    file = new Gif( parent, filename);
    frames = Gif.getPImages( parent, filename );
    w = frames[0].width;
    h = frames[0].height;
    bLoaded = true;
  }
  public void play() {
    file.play();
  }
  
  public void render() {
    if(bActive) {
      fill(255, 0, 0);
      rect(x-2, y-2, w+4, h+4);
      frameIndex = (int)map(mousePct, 0.0f, 1.0f, 0, frames.length);
      if(frameIndex > frames.length-1) frameIndex = frames.length-1;
      image(frames[frameIndex], x, y );
    } else {
      fill(255);
      image( file, x, y);
    }
    
    if(bActive) {
      fill(255);
      noStroke();
      float rx = map( mousePct, 0.0f, 1.0f, x, x + w-3);
      rect(rx, y, 3, h);
    }
  }
  
  public Boolean mouseOver( float mx, float my ) {
    if(mx >= x && mx <= x + w) {
      if(my >= y && my <= y + h) {
        bActive = true;
        mousePct = (mx - x) / w;
        if(mousePct > 1.0f) mousePct = 1.0f;
        if(mousePct < 0.0f) mousePct = 0.0f;
        return true;
      }
    }
    bActive = false;
    return false;
  }
};
  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#FFFFFF", "gif_recorder" });
  }
}
