// http://www.extrapixel.ch/processing/gifAnimation/
import gifAnimation.*;
import processing.video.*;

// Variable for capture device
Capture video;
GifMaker gifExport;
InteractiveGif gifPlayer;

Boolean bRecording = false;
Boolean bLiveView = true;
Boolean bDragged = false;
int numFrames = 0;
String instructions = "Click to record";

void setup() {
  size(320, 240);
  frameRate( 10 );
  textFont(createFont("Arial", 18));
  // Uses the default video input, see the reference if this causes an error
  video = new Capture(this, width, height);
  video.start();
}

void draw() {
  background(0);
  if(!bLiveView && !bRecording) {
    if(gifPlayer.bLoaded) {
      if(bDragged) {
        gifPlayer.mouseOver( mouseX, mouseY );
      } else {
        gifPlayer.bActive = false; 
      }
      gifPlayer.render();
    }
  }
  
  if (bLiveView && video.available()) {
    video.read();
    video.loadPixels(); 
    image( video, 0, 0 );
    if(bRecording) {
      gifExport.setDelay(1);
      gifExport.addFrame();
      numFrames ++;
      //fill(255, 0, 0);
      //rect(0,0,34,22);
      //fill(255);
      //text(Integer.toString(numFrames), 8, 18 );
      instructions = "Click to finish - "+Integer.toString(numFrames);
    }
  }
  fill(0);
  if(bRecording) fill(255, 0, 0);
  float textw = textWidth(instructions)+8;
  rect( 4, height - 32, textw, 22);
  fill(255);
  text(instructions, 8, height - 14 );
}

void stop() {
  if(bRecording) {
    gifExport.finish();
  }
  super.stop();
}

void mouseDragged() {
  println("mouse dragged");
  bDragged = true;
}

void mouseClicked() {
  println("mouse pressed");
  
  if(!bDragged) {
    if(!bLiveView && !bRecording) {
      bLiveView = true;
      instructions = "Click to record";
      //return;
    } else if(bRecording) {
      bRecording = false;
      bLiveView = false;
      instructions = "Click and drag to scrub / click to record";
      gifExport.finish();
      gifPlayer.load(this, gifPlayer.filename );
      gifPlayer.play();
    } else {
      instructions = "Click to finish";
      createExporter();
      bRecording = true;
    }
  }
  bDragged = false;
}

void mouseReleased() {
  println("mouse released");
  bDragged = false;
}

void createExporter() {
  String uniqueID = "gifs/export_"+getUniqueID()+".gif";
  gifExport = new GifMaker(this, uniqueID);
  gifExport.setRepeat(0); // make it an "endless" animation
  gifPlayer = new InteractiveGif();
  gifPlayer.filename = uniqueID;
  
  numFrames = 0;
}

String getUniqueID() {
  return Integer.toString(month()+day()+year()+hour()+minute()+second());
}
